

float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matWorldView;

float4 vecSunDir;		
float4 vecViewPos;

float4 vecLightPos[8]; //light position
float4 vecLightColor[8]; //light color
float4 vecSunColor;

texture entSkin1;
texture entSkin2;
texture entSkin3;


sampler ColorMapSampler = sampler_state
{
	Texture   = <entSkin1>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;   
	AddressU  = wrap;
	AddressV  = wrap;
};

sampler BumpMapSampler = sampler_state
{
	Texture   = <entSkin2>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;   
	AddressU  = wrap;
	AddressV  = wrap;
};

sampler RelaxedConeMapSampler = sampler_state
{
	Texture   = <entSkin3>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;   
	AddressU  = wrap;
	AddressV  = wrap;
};



void DiffuseVS(		
in float4 vPos		   : POSITION,
in float3 normal	   : NORMAL,
in float2 InTex	   : TEXCOORD0,
in float4 InTangent	: TEXCOORD2,


out float2 OutTex	      : TEXCOORD0,			
out float3 OutSunDir    : TEXCOORD1,
out float3 View	      : TEXCOORD2,

out float4 Light1 		: TEXCOORD3,
out float4 Light2 		: TEXCOORD4,
out float4 Light3 		: TEXCOORD5,
out float4 Light4 		: TEXCOORD6,

out float4 Pos	         : POSITION
)
{

	Pos = mul(vPos,matWorldViewProj);
	
	///////////////////////////////////////////////
	float3x3 modelViewRotXf;
	modelViewRotXf[0] = matWorld[0].xyz;
	modelViewRotXf[1] = matWorld[1].xyz;
	modelViewRotXf[2] = matWorld[2].xyz;	
	///////////////////////////////////////////////
	
	
	float3 binormal = cross(InTangent.xyz,normal)*InTangent.w;

	float3x3 Tangentspace;

	Tangentspace[0] = mul(InTangent.xyz,matWorld);	
	Tangentspace[1] = mul(binormal,matWorld);
	Tangentspace[2] = mul(normal,matWorld);



	OutTex          = InTex;

	float3 world    = mul(vPos,matWorld);

	View =  vecViewPos -world ;//

	View = mul(Tangentspace,View);

	OutSunDir       = normalize(mul(Tangentspace, -vecSunDir));


	//light 1	 
	Light1.xyz = world - vecLightPos[0]  ; 
	Light1.xyz = mul(Tangentspace, -Light1.xyz);	// L
	Light1.w= distance(world,vecLightPos[0])/vecLightPos[0].w;	// Point light
	
	//light 2	 
	Light2.xyz = world - vecLightPos[1]  ; 
	Light2.xyz = mul(Tangentspace, -Light2.xyz);	// L
	Light2.w= distance(world,vecLightPos[1])/vecLightPos[1].w;	// Point light
	
	
	//light 3	 
	Light3.xyz = world - vecLightPos[2]  ; 
	Light3.xyz = mul(Tangentspace, -Light3.xyz);	// L
	Light3.w= distance(world,vecLightPos[2])/vecLightPos[2].w;	// Point light

	//light 4	 
	Light4.xyz = world - vecLightPos[3]  ; 
	Light4.xyz = mul(Tangentspace, -Light4.xyz);	// L
	Light4.w= distance(world,vecLightPos[3])/vecLightPos[3].w;	// Point light
}



float4 DiffusePS(	

in float2 InTex		     : TEXCOORD0,
in float3 Sun     	     : TEXCOORD1,     
in float3 View		        : TEXCOORD2,

in float4 Light1 			  : TEXCOORD3,
in float4 Light2 			  : TEXCOORD4,
in float4 Light3 		     : TEXCOORD5,
in float4 Light4 		     : TEXCOORD6


) : COLOR
{

	
	View      = normalize(View);


	// \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
	// \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\RELAXED CONE MAPPING based on Fabio Policarpos Shader: http://fabio.policarpo.nom.br/  

	const int effect_size=3.f;
	const int cone_steps=15;
	const int binary_steps=8;
	
//	float limit=saturate(length(View.xy/View.z)/15);

	float3 v;
	v=float3(View.x,View.y,-View.z*10);//	v=float3(View.x,View.y,-View.z*10/effect_size-limit);
	
	float3 p;
	p=float3(InTex.xy,0);
	
	float3 p0=p;
	
	v /=v.z;
	
	float dist = length(v.xy);		


	for( int i=0;i<cone_steps;i++ )
	{
		float4 tex = tex2D(RelaxedConeMapSampler, p.xy);

		float height = saturate((tex.w) - p.z );
		
		float cone_ratio = tex.z;
		
		p += (v * ((cone_ratio * height) / (dist + cone_ratio)));
	}

	v *= p.z*0.5;
	p = p0 + v;

	for( int i=0;i<binary_steps;i++ )
	{
		float4 tex = tex2D(RelaxedConeMapSampler, p.xy);
		v *= 0.5;
		if (p.z<tex.w)
		p+=v;
		else
		p-=v;
	}

	float2 Tex=p.xy;

	// //////////////////////////////////////////////////////
	// ////////////////////////////////////////////////////


	float4 c,b;	
	c = tex2D(ColorMapSampler,Tex);	
	b = tex2D(BumpMapSampler,Tex)*2-1;
	
	b.xy*=-1;

	//sun_light
	Sun        = normalize(Sun);	
	float diff_sun      = saturate(dot(b,Sun));
	float shadow_sun      = saturate(4 * diff_sun);
	float3 Reflect_sun    = normalize(2 * diff_sun * b - Sun); 
	float4 spec_sun       = 0.8f * pow(saturate(dot(Reflect_sun, View)),20);




	//light1
	float3 LightDir1 = normalize(Light1.xyz);   
	float diff1 = saturate(dot(b, LightDir1));    // diffuse component 
	float shadow1 = saturate(4 * diff1);
	float3 Reflect1 = normalize(2 * diff1 * b - LightDir1);  // R
	float4 spec1 = 0.8f * pow(saturate(dot(Reflect1, View)), 20); 
	float4 Attenuation1 = saturate(dot(Light1.w, Light1.w));
	
	//light2
	float3 LightDir2 = normalize(Light2.xyz);   
	float diff2 = saturate(dot(b, LightDir2));    // diffuse component 
	float shadow2 = saturate(4 * diff2);
	float3 Reflect2 = normalize(2 * diff2 * b - LightDir2);  // R
	float4 spec2 = 0.8f * pow(saturate(dot(Reflect2, View)), 20); 
	float4 Attenuation2 = saturate(dot(Light2.w, Light2.w));
	
	//light3
	float3 LightDir3 = normalize(Light3.xyz);   
	float diff3 = saturate(dot(b, LightDir3));    // diffuse component 
	float shadow3 = saturate(4 * diff3);
	float3 Reflect3 = normalize(2 * diff3 * b - LightDir3);  // R
	float4 spec3 = 0.8f * pow(saturate(dot(Reflect3, View)), 20); 
	float4 Attenuation3 = saturate(dot(Light3.w, Light3.w));

	//light4
	float3 LightDir4 = normalize(Light4.xyz);   
	float diff4 = saturate(dot(b, LightDir4));    // diffuse component 
	float shadow4 = saturate(4 * diff4);
	float3 Reflect4 = normalize(2 * diff4 * b - LightDir4);  // R
	float4 spec4 = 0.8f * pow(saturate(dot(Reflect4, View)), 20); 
	float4 Attenuation4 = saturate(dot(Light4.w, Light4.w));


	c.rgb =(shadow_sun * (c * diff_sun + (spec_sun*c.w) * vecSunColor))
	+
	((shadow1 * (c * diff1 + (spec1*c.w)) * (1 -Attenuation1))*vecLightColor[0])+
	((shadow2 * (c * diff2 + (spec2*c.w)) * (1 -Attenuation2))*vecLightColor[1])+
	((shadow3 * (c * diff3 + (spec3*c.w)) * (1 -Attenuation3))*vecLightColor[2])+
	((shadow4 * (c * diff4 + (spec4*c.w)) * (1 -Attenuation4))*vecLightColor[3]);
	
	return c;
}

DWORD ref=128.0f;
technique DiffuseTechnique
{
	pass P0
	{
		alphablendenable=false;
		alphatestenable=true;
		AlphaRef=<ref>;
		AlphaFunc=Greater;
		zenable=true;
		zwriteenable=true;

		VertexShader = compile vs_3_0 DiffuseVS();
		PixelShader  = compile ps_3_0 DiffusePS();
	}
}